/////////////////////////////////////////////////////////////////////////////////////////////
// copy this folder inside your gamestudio folder
// open carai3.wmp, build it and run it using carai3.wdl
/////////////////////////////////////////////////////////////////////////////////////////////

var video_mode = 7; // 800x600 pixels
var video_depth = 16; // 16 bit mode

var car1_time; // current racing time for car1
var car2_time; // current racing time for car2
var car3_time; // current racing time for car3

var place = 0; 

/////////////////////////////////////////////////////////////////////////////////////////////

string carai3_wmb = <carai3.wmb>;

string temp_str; // just a temporary string
string car1_str;
string car2_str;
string car3_str;

/////////////////////////////////////////////////////////////////////////////////////////////

bmap finish_pcx = <finish.pcx>;

/////////////////////////////////////////////////////////////////////////////////////////////

sound beep_sound = <beep.wav>;
sound engine_wav = <engine.wav>;
sound lap_wav = <lap.wav>; // played at the end of every lap
sound horn_wav = <horn.wav>;

/////////////////////////////////////////////////////////////////////////////////////////////

define race_over skill20; // will be set to 1 when the race if over
define counting_laps skill21; // enables / disables the counter for the number of laps
define laps skill22; // holds the number of laps
define max_speed skill23; // maximum speed for a specific car, changes every lap
define min_speed skill24; // minimum speed for a specific car, changes every lap
define distance skill25; // distance covered by the car
define scans skill26; // number of scans for a specific car
define car_speed skill27; // uses skills 27..29

/////////////////////////////////////////////////////////////////////////////////////////////

function add_laps();
function make_way();

/////////////////////////////////////////////////////////////////////////////////////////////

font system_font = <system.pcx>, 10, 12; 

/////////////////////////////////////////////////////////////////////////////////////////////

entity* car1; // needed only for the panels
entity* car2;
entity* car3;

/////////////////////////////////////////////////////////////////////////////////////////////

include <playercar.wdl>;

/////////////////////////////////////////////////////////////////////////////////////////////

text car1_txt
{
	font = system_font;
	pos_x = 180;
	pos_y = 0;
	string = car1_str;	
	layer = 15;
}

text car2_txt
{
	font = system_font;
	pos_x = 180;
	pos_y = 0;
	string = car2_str;	
	layer = 15;
}

text car3_txt
{
	font = system_font;
	pos_x = 180;
	pos_y = 0;
	string = car3_str;	
	layer = 15;
}

panel finish_pan
{
	bmap = finish_pcx;
	pos_x = 100;
	pos_y = 100;
	alpha = 50;
	layer = 10;
	flags = overlay, refresh, transparent;
}

function main()
{
	randomize();
	time_smooth = 0.75;
	fps_max = 30; // limit the frame rate to 30 fps
	clip_size = 0; // show all the triangles for all the models (needed only for A5)
	level_load (carai3_wmb);
	wait (3);
	while (1)
	{	
/*		if (car1.race_over == 0) // race isn't over for car1?
		{
			car1_time = total_frames / 30; // compute its racing time
		}
		if (car2.race_over == 0) // same thing for car2
		{
			car2_time = total_frames / 30;
		}
		if (car3.race_over == 0) // and car3
		{
			car3_time = total_frames / 30;
		}*/
		wait (1);
	}
}

action car
{
	var waypoint_number = 0;
	var target_coords; // the coordinates of the next waypoint on the path
	var engine_handle;
	var angle_diff;
	var engine_offset; // creates a random (unique) engine sound for every car
	my.race_over = 0;
	my.counting_laps = 1;
	my.laps = -1;
	my.max_speed = 50; // initial value needed to start the car
	my.min_speed = 50; // initial value needed to start the car
	my.enable_scan = on; // the car is sensitive to scanning
	my.event = make_way; // its event function will run if it is scanned
	engine_offset = random(3); // generate a random value that will be used together with snd_tune to generate a unique engine sound
	if (my.skill1 == 1) // don't forget to set skill1 = 1 for the first car, skill1 = 2 for the second car and so on
	{
		ent_path("car01"); // if skill1 = 1 the car will follow the path named "car01" without the quotes
		car1 = me; // I'm car 1
	}
	if (my.skill1 == 2) // same thing for the rest of the cars
	{
		ent_path("car02");
		car2 = me;
	}
	if (my.skill1 == 3)
	{
		ent_path("car03");
		car3 = me;
	}
	engine_handle = ent_playloop (my, engine_wav, 50); // play the engine sound in a loop
	ent_waypoint(target_coords, 1); // set the target coordinates to the first point on the path
	while (my.race_over == 0) // this loop will run until my.race_over = 1
	{
		temp.x = 60; // horizontal scanning angle 
		temp.y = 60; // vertical scanning angle
		temp.z = 150; // scanning range
		scan_entity (my.x, temp); // scan the cars in front of me on a distance of 150 quants

		temp.x = target_coords.x - my.x; // get the vector that points towards the target
		temp.y = target_coords.y - my.y;
		temp.z = 0;
		if (vec_to_angle (angle_diff.pan, temp) < 150) // closer than 150 quants to the target?
		{
			waypoint_number = ent_nextpoint(target_coords); // move towards the new point on the path
		}
		angle_diff.pan = ang(angle_diff.pan - my.pan); // get the difference angle
		my.pan += my.car_speed * angle_diff.pan * 0.005; // change the pan angle of the car slowly
		if (((waypoint_number > 65) || (waypoint_number < 8)) || ((waypoint_number > 34) && (waypoint_number < 49))) // horizontal areas of the track?
		{
			if (my.car_speed < (my.max_speed * time)) // and the speed is below max_speed?
			{
				my.car_speed += 1 * time; // increase the speed of the car
			}
		}
		else // curved areas of the track
		{
			if (my.car_speed > (my.min_speed * time)) // and the speed is above min_speed?
			{
				my.car_speed -= 0.5 * time; // decrease the speed of the car
			}
		}
		my.skill28 = 0; // second skill used for car_speed, we are using skill27..skill29 (skill27 was defined as car_speed)
		my.skill29 = 0; // third skill used for car_speed
		move_mode = ignore_you + ignore_passable + glide;
		my.skill2 = ent_move (my.car_speed, nullskill); // store the distance covered by this car in skill2
		my.distance += my.skill2; // and store the total distance in my.distance (know which car is in front at a certain moment, etc)
		snd_tune (engine_handle, 50, (my.skill2 * 5 + engine_offset), 0); // tune the sound depending on the "real" speed of the car (distance covered)

		vec_set (temp, my.x);
		temp.z -= 1000;
		trace_mode = ignore_me + ignore_passable + scan_texture;
		trace (my.x, temp); // trace 1000 quants below the car
		if (str_cmpi ("checker1", tex_name)) // if we've got the texture named "checker1" (without the quotes)
		{		
			add_laps(); // increase the number of laps
		}
		if (my.laps == 3) // end of race?
		{
			my.race_over = 1; // get out of the while loop after 3 laps
			place += 1;
			if (place == 1) // i'm the first car that finishes the race?
			{
				if (my.skill1 == 1)
				{
					str_cpy (car1_str, "1st place: Green car            Time: "); 
					str_for_num (temp_str, car1_time);
					str_cat (car1_str, temp_str);
					car1_txt.pos_y = 250;
					car1_txt.visible = on;
				}
				if (my.skill1 == 2)
				{
					str_cpy (car2_str, "1st place: Red car              Time: "); 
					str_for_num (temp_str, car2_time);
					str_cat (car2_str, temp_str);
					car2_txt.pos_y = 250;
					car2_txt.visible = on;
				}
				if (my.skill1 == 3)
				{
					str_cpy (car3_str, "1st place: Blue car             Time: "); 
					str_for_num (temp_str, car3_time);
					str_cat (car3_str, temp_str);
					car3_txt.pos_y = 250;
					car3_txt.visible = on;
				}
			}
			if (place == 2) // i'm the second car that finishes the race?
			{
				if (my.skill1 == 1)
				{
					str_cpy (car1_str, "2nd place: Green car            Time: "); 
					str_for_num (temp_str, car1_time);
					str_cat (car1_str, temp_str);
					car1_txt.pos_y = 280;
					car1_txt.visible = on;
				}
				if (my.skill1 == 2)
				{
					str_cpy (car2_str, "2nd place: Red car              Time: "); 
					str_for_num (temp_str, car2_time);
					str_cat (car2_str, temp_str);
					car2_txt.pos_y = 280;
					car2_txt.visible = on;
				}
				if (my.skill1 == 3)
				{
					str_cpy (car3_str, "2nd place: Blue car             Time: "); 
					str_for_num (temp_str, car3_time);
					str_cat (car3_str, temp_str);
					car3_txt.pos_y = 280;
					car3_txt.visible = on;
				}
			}
			if (place == 3) // i'm the third car that finishes the race?
			{
				if (my.skill1 == 1)
				{
					str_cpy (car1_str, "3rd place: Green car            Time: "); 
					str_for_num (temp_str, car1_time);
					str_cat (car1_str, temp_str);
					car1_txt.pos_y = 310;
					car1_txt.visible = on;
				}
				if (my.skill1 == 2)
				{
					str_cpy (car2_str, "3rd place: Red car              Time: "); 
					str_for_num (temp_str, car2_time);
					str_cat (car2_str, temp_str);
					car2_txt.pos_y = 310;
					car2_txt.visible = on;
				}
				if (my.skill1 == 3)
				{
					str_cpy (car3_str, "3rd place: Blue car             Time: "); 
					str_for_num (temp_str, car3_time);
					str_cat (car3_str, temp_str);
					car3_txt.pos_y = 310;
					car3_txt.visible = on;
				}
			}
			finish_pan.visible = on;
		}
		wait(1);
	}
	while (my.car_speed > 1) // decrease the speed from its current speed to 1
	{
		my.car_speed -= 1 * time;
		snd_tune (engine_handle, 50, my.car_speed * 5, 0); // and tune the sound accordingly
		my.skill28 = 0;
		my.skill29 = 0;
		move_mode = ignore_you + ignore_passable + glide;
		ent_move (my.car_speed, nullskill); 
		wait (1);
	}
	snd_stop (engine_handle); // stop the engine sound
	if (place == 3) // race finished for all the cars?
	{
		while (finish_pan.alpha < 95)
		{
			finish_pan.alpha += 2 * time; // increase alpha
			wait (1);
		}
		finish_pan.transparent = off; // the panel is opaque now
	}
}

function add_laps()
{
	if (my.counting_laps == 0) {return;} // lap added already? return
	my.max_speed = 50 + random(10); // get a new max_speed value every lap
	my.min_speed = 30 + random(10); // get a new min_speed value every lap
	my.laps += 1; // increase the number of laps
	snd_play (lap_wav, 100, 0); // play a sound
	my.counting_laps = 0; // and stop adding laps
	sleep (10); // for 10 seconds (don't register multiple successful traces for a single lap)
	my.counting_laps = 1; // now you can add laps again
}

function make_way()
{
	my.scans += 1; // increase the number of scan
	if (you.car_speed >= 5) // the speed of the car is above 5?
	{
		you.car_speed -= 0.5 * time; // decrease the speed of the car that has scanned me. Play with 0.5
	}
	if (my.scans > 5) // too many scans? play with "5"
	{ 
		ent_playsound (my, horn_wav, 2000); // I'll use my horn then
		my.enable_scan = off; // stop reacting to scans
		sleep (1); // for one second
		my.enable_scan = on; // enable scans again
		my.scans = 0; // reset the number of scans (prepare for a new horn)
	}
}